function[res_savgol,plage]=savgol(saisir,plage,varargin)
//
// Algorithme basé sur Savitsky-Golay (SG), NON modifiée cad projection 
// sur trois axes non orthogonaux entre eux
//
// ------------------------------------------------------------------------
// Entrées: 
//    Xsaisir: matrice au format Saisir ou bien matrice classique 2D
//    plage: nombre entier positif impair
// Sorties:
//    res_savgol.d0.d      spectres lissés, dérivée 0
//    res_savgol.d1.d      spectres dérivée première
//    res_savgol.d2.d      spectres dérivée seconde
// ------------------------------------------------------------------------
//
// Avertissement: 
// SG ne gère pas les demi-plages à droite et à gauche. Celles-ci 
// sont approximées au mieux par régression aux moindres carrés, mais le 
// résultat est toujours nettement moins bien que lorsque SG est applicable.
// En conséquence les extrémités droite et gauche sont perturbées,
// et la perturbation s'amplifie avec le degré de dérivation
// ------------------------------------------------------------------------


////Vérification/transformation en matrice Saisir:
//if type(saisir)==1 
//         saisir.d=saisir;
//         [n,p]=size(saisir.d);
//         n1=[1:1:n];
//         p1=[1:1:p];
//         saisir.i=string(n1');
//         saisir.v=string(p1');
//else     if type(saisir)==17
//                 test=1;
//                 [n,p]=size(saisir.d);
//                 [ni,pi]=size(saisir.i);
//                 [nv,pv]=size(saisir.v)
//                 test=abs(n-max(ni,pi))+abs(p-max(nv,pv))+abs(min(ni,pi)-1)+abs(min(nv,pv)-1);
//                    if(test>0) error('Verify your Saisir matrix'); end
//                    if ni<pi  saisir.i=saisir.i'; end
//                    if nv<pv  saisir.v=saisir.v'; end
//         else  error('data is neither a matrix nor a Saisir structure')         
//         end
//end   

saisir=div(saisir);
                 
//vérification de l'absence de valeurs manquantes
test=sum(sum(isnan(saisir.d)));
if(test>0)
        xdisp('There are ',test,' Nan values in the data table');
        xdisp('It is necessary to resolve the problem before using pls-cv');
        error(' ');
end


// choix du degré du polynome
if argn(2)==2 then
    degre=2+1;       // degré 2
else degre=varargin(1)+1;
end

if degre<3 | degre>6 then
    error('the degree of the polynom should be between 2 and 5')
end


//Initialisation:
[n,p]=size(saisir.d);

// verification ajustement; plage est un entier impair 
plage=round(plage);
if plage<=4 then error('la plage doit etre un entier superieur ou egal a 5');end
  
rayon=floor((plage-1)/2);
plage=2*rayon+1;

d0_savgol=saisir.d;
d1_savgol=ones(n,p);
d2_savgol=ones(n,p);
m=zeros(6,plage);



// calculs de Savitsky-Golay sur le spectre, cotés exceptés 

//Calcul de m  
vecteur=[-rayon:1:rayon];
m(1,:)=vecteur.^0;  
m(2,:)=vecteur.^1;  
m(3,:)=vecteur.^2;  
m(4,:)=vecteur.^3;  
m(5,:)=vecteur.^4;  
m(6,:)=vecteur.^5;  


// ajustement de m à la dimension du polynome
m=m(1:degre,:);

c=inv(m*m')*m; 

for a=rayon+1:p-rayon-1;
      d0_savgol(:,a)=saisir.d(:,a-rayon:a+rayon)*c(1,:)';
      d1_savgol(:,a)=saisir.d(:,a-rayon:a+rayon)*c(2,:)';
      d2_savgol(:,a)=2*saisir.d(:,a-rayon:a+rayon)*c(3,:)';
      //la multiplication par 2 correspond à la dérivée seconde de: a+bx+cx**2
end;

// calculs sur les cotés du spectre (non Savitsky-Golay)

//d0 à gauche
  calcul=saisir.d(:,1:2*rayon+1)*c'*m;
  d0_savgol(:,1:rayon)=calcul(:,1:rayon);
//d0 à droite
  calcul=saisir.d(:,p-2*rayon:p)*c'*m;
  d0_savgol(:,p-rayon:p)=calcul(:,rayon+1:2*rayon+1);

//d1 à gauche
  d1=d0_savgol(:,2:p)-d0_savgol(:,1:p-1);
  d1_savgol(:,1:rayon)=d1(:,1:rayon);
  calcul=d1_savgol(:,1:2*rayon+1)*m'*inv(m*m')*m;
  d1_savgol(:,1:rayon)=calcul(:,1:rayon);
//d1 à droite
  d1_savgol(:,p-rayon:p)=d1(:,p-rayon-1:p-1);   //perte d'un indice à cause de la dérivée première
  calcul=d1_savgol(:,p-2*rayon:p)*m'*inv(m*m')*m;
  d1_savgol(:,p-rayon:p)=calcul(:,rayon+1:2*rayon+1);

//d2 à gauche
  d2=d1_savgol(:,2:p-1)-d1_savgol(:,1:p-2);
  d2_savgol(:,1:rayon)=d2(:,1:rayon);
  calcul=d2_savgol(:,1:2*rayon+1)*m'*inv(m*m')*m;
  d2_savgol(:,1:rayon)=calcul(:,1:rayon);
//d2 à droite
  d2_savgol(:,p-rayon:p)=d2(:,p-rayon-2:p-2);   //perte d'un indice à cause de la dérivée seconde
  calcul=d2_savgol(:,p-2*rayon:p)*m'*inv(m*m')*m;
  d2_savgol(:,p-rayon:p)=calcul(:,rayon+1:2*rayon+1);


//Résultats Div:

res_savgol.d0.d=d0_savgol;
res_savgol.d0.v=saisir.v;
res_savgol.d0.i=saisir.i;
res_savgol.d0=div(res_savgol.d0);

res_savgol.d1.d=d1_savgol;
res_savgol.d1.v=saisir.v;
res_savgol.d1.i=saisir.i;
res_savgol.d1=div(res_savgol.d1);

res_savgol.d2.d=d2_savgol;
res_savgol.d2.v=saisir.v;
res_savgol.d2.i=saisir.i;
res_savgol.d2=div(res_savgol.d2);

endfunction;

